@@ -1,7 +1,7 @@
 from .main import get_os_client, get_df_client, os_upload, os_upload_json, \
     os_download, os_download_json, os_download_to_memory, dfapp_get_os_client, \
     os_delete_objects, os_delete_object_if_exists, os_delete_object, get_delegation_token, \
-    os_get_endpoint, os_rename_objects, os_rename_object, os_has_object, list_objects_start_with, \
+    os_rename_objects, os_rename_object, os_has_object, list_objects_start_with, \
     get_secrets_client, get_vaults_client, secrets_read_value, secrets_read_value_by_name, \
     secrets_list_secrets, secrets_create_value, secrets_update_value, secrets_delete_value, \
-    secrets_undelete_value, secrets_create_or_update_value
+    secrets_undelete_value, secrets_create_or_update_value, get_region_name, os_copy_object
